@@ -39,6 +39,7 @@
 
 #include <gnome.h>
 #include <gtk/gtklabel.h>
+#include <gtk/gtkmessagedialog.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
 #include <libgnomevfs/gnome-vfs-find-directory.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
@@ -1098,7 +1099,7 @@ handle_transfer_overwrite (const GnomeVFSXferProgressInfo *progress_info,
 	is_merge =  target_is_dir && is_directory (progress_info->source_name);
 
 	if (is_merge) {
-		secondary_text = _("If you replace the existing folder, any files in it that conflicts with the files being copied will be overwritten.");
+		secondary_text = _("If you replace the existing folder, any files in it that conflict with the files being copied will be overwritten.");
 	} else {
 		secondary_text = _("If you replace an existing file, its contents will be overwritten.");
 	}
@@ -1837,6 +1838,7 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 			have_nonlocal_source = TRUE;
 		}
 			
+		/* Note: this could be null if we're e.g. copying the top level file of a web site */
 		source_dir_uri = gnome_vfs_uri_get_parent (source_uri);
 		target_uri = NULL;
 		if (target_dir != NULL) {
@@ -1861,12 +1863,14 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 			target_uri_list = g_list_prepend (target_uri_list, target_uri);
 			source_uri_list = g_list_prepend (source_uri_list, source_uri);
 
-			if (duplicate
-			    && !gnome_vfs_uri_equal (source_dir_uri, target_dir_uri)) {
+			if (duplicate && source_dir_uri != NULL &&
+			    !gnome_vfs_uri_equal (source_dir_uri, target_dir_uri)) {
 				duplicate = FALSE;
 			}
 		}
-		gnome_vfs_uri_unref (source_dir_uri);
+		if (source_dir_uri != NULL) {
+			gnome_vfs_uri_unref (source_dir_uri);
+		}
 	}
 
 	if (target_is_trash) {
@@ -1926,7 +1930,7 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 		/* localizers: progress dialog title */
 		transfer_info->operation_title = _("Moving files to the Trash");
 		/* localizers: label prepended to the progress count */
-		transfer_info->action_label =_("Files thrown out:");
+		transfer_info->action_label =_("Throwing out file:");
 		/* localizers: label prepended to the name of the current file moved */
 		transfer_info->progress_verb =_("Moving");
 		transfer_info->preparation_name =_("Preparing to Move to Trash...");
@@ -1937,7 +1941,7 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 		/* localizers: progress dialog title */
 		transfer_info->operation_title = _("Moving files");
 		/* localizers: label prepended to the progress count */
-		transfer_info->action_label =_("Files moved:");
+		transfer_info->action_label =_("Moving file:");
 		/* localizers: label prepended to the name of the current file moved */
 		transfer_info->progress_verb =_("Moving");
 		transfer_info->preparation_name =_("Preparing To Move...");
@@ -1951,7 +1955,7 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 		/* localizers: progress dialog title */
 		transfer_info->operation_title = _("Creating links to files");
 		/* localizers: label prepended to the progress count */
-		transfer_info->action_label =_("Files linked:");
+		transfer_info->action_label =_("Linking file:");
 		/* localizers: label prepended to the name of the current file linked */
 		transfer_info->progress_verb =_("Linking");
 		transfer_info->preparation_name = _("Preparing to Create Links...");
@@ -1963,7 +1967,7 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 		/* localizers: progress dialog title */
 		transfer_info->operation_title = _("Copying files");
 		/* localizers: label prepended to the progress count */
-		transfer_info->action_label =_("Files copied:");
+		transfer_info->action_label =_("Copying file:");
 		/* localizers: label prepended to the name of the current file copied */
 		transfer_info->progress_verb =_("Copying");
 		transfer_info->preparation_name =_("Preparing To Copy...");
@@ -2457,11 +2461,12 @@ nautilus_file_operations_delete (const GList *item_uris,
 
 					link = nautilus_desktop_icon_file_get_link (NAUTILUS_DESKTOP_ICON_FILE (file));
 
-					nautilus_desktop_link_monitor_delete_link (nautilus_desktop_link_monitor_get (),
-										   link,
-										   parent_view);
-					
-					g_object_unref (link);
+					if (link != NULL) {
+						nautilus_desktop_link_monitor_delete_link (nautilus_desktop_link_monitor_get (),
+											   link,
+											   parent_view);
+						g_object_unref (link);
+					}
 				}
 				nautilus_file_unref (file);
 			}
@@ -2541,10 +2546,9 @@ static gboolean
 confirm_empty_trash (GtkWidget *parent_view)
 {
 	GtkWidget *dialog;
-	int response;
-	GtkWidget *hbox, *vbox, *image, *label, *button;
-	gchar     *str;
+	GtkWidget *button;
 	GdkScreen *screen;
+	int response;
 
 	/* Just Say Yes if the preference says not to confirm. */
 	if (!eel_preferences_get_boolean (NAUTILUS_PREFERENCES_CONFIRM_TRASH)) {
@@ -2553,13 +2557,23 @@ confirm_empty_trash (GtkWidget *parent_view)
 	
 	screen = gtk_widget_get_screen (parent_view);
 
-	dialog = gtk_dialog_new ();
+	dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+					 GTK_MESSAGE_WARNING,
+					 GTK_BUTTONS_NONE,
+					 _("Empty all of the items from "
+					   "the trash?"));
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						  _("If you choose to empty "
+						    "the trash, all items in "
+						    "it will be permanently "
+						    "lost. Please note that "
+						    "you can also delete them "
+						    "separately."));
+
+	gtk_window_set_title (GTK_WINDOW (dialog), ""); /* as per HIG */
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), TRUE);
 	gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 	atk_object_set_role (gtk_widget_get_accessible (dialog), ATK_ROLE_ALERT);
-	gtk_window_set_title (GTK_WINDOW (dialog), "");
-	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 	gtk_window_set_wmclass (GTK_WINDOW (dialog), "empty_trash",
 				"Nautilus");
 
@@ -2567,55 +2581,11 @@ confirm_empty_trash (GtkWidget *parent_view)
         gtk_widget_realize (dialog);
 	gdk_window_set_transient_for (GTK_WIDGET (dialog)->window,
 				      gdk_screen_get_root_window (screen));
-	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-	
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 14);
-
-	hbox = gtk_hbox_new (FALSE, 12);
-	gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
-	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox,
-			    FALSE, FALSE, 0);
-
-	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_QUESTION,
-					  GTK_ICON_SIZE_DIALOG);
-	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
-	gtk_widget_show (image);
-	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
-
-	vbox = gtk_vbox_new (FALSE, 12);
-	gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
-	gtk_widget_show (vbox);
-
-	str = g_strconcat ("<span weight=\"bold\" size=\"larger\">", 
-		_("Are you sure you want to empty "
-		"all of the items from the trash?"), 
-		"</span>", 
-		NULL);
-		
-	label = gtk_label_new (str);  
-	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
-	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
-	gtk_widget_show (label);
-	g_free (str);
-
-	label = gtk_label_new (_("If you empty the trash, items "
-		"will be permanently deleted."));
-	
-	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
-	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
-	gtk_widget_show (label);
 
 	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL,
 			       GTK_RESPONSE_CANCEL);
 
-	button = eel_gtk_button_new_with_stock_icon (_("_Empty"),
-						     GTK_STOCK_DELETE);
+	button = gtk_button_new_with_mnemonic (_("_Empty Trash"));
 	gtk_widget_show (button);
 	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
 
