@@ -235,6 +235,30 @@ update_desktop_dir (void)
 	gnome_vfs_uri_unref (vfs_uri);
 }
 
+gboolean
+nautilus_is_home_directory_file_escaped (char *escaped_dirname,
+					 char *escaped_file)
+{
+	static char *escaped_home_dir_dirname = NULL;
+	static char *escaped_home_dir_filename = NULL;
+	char *uri;
+	GnomeVFSURI *vfs_uri;
+	
+	if (escaped_home_dir_dirname == NULL) {
+		uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
+		vfs_uri = gnome_vfs_uri_new (uri);
+		g_free (uri);
+
+		escaped_home_dir_filename = gnome_vfs_uri_extract_short_path_name (vfs_uri);
+		escaped_home_dir_dirname = gnome_vfs_uri_extract_dirname (vfs_uri);
+
+		gnome_vfs_uri_unref (vfs_uri);
+	}
+
+	return (strcmp (escaped_dirname, escaped_home_dir_dirname) == 0 &&
+		strcmp (escaped_file, escaped_home_dir_filename) == 0);
+}
+					 
 gboolean
 nautilus_is_desktop_directory_file_escaped (char *escaped_dirname,
 					    char *escaped_file)
@@ -342,6 +366,55 @@ nautilus_get_data_file_path (const char *partial_path)
 	return NULL;
 }
 
+static gboolean
+test_uri_exists (const char *path)
+{
+	GnomeVFSURI *uri;
+	gboolean exists;
+
+	uri = gnome_vfs_uri_new (path);
+	exists = gnome_vfs_uri_exists (uri);
+	gnome_vfs_uri_unref (uri);
+
+	return exists;
+}
+
+char *
+nautilus_ensure_unique_file_name (const char *directory_uri,
+				  const char *base_name,
+				  const char *extension)
+{
+	char *path, *escaped_name;
+	gboolean exists;
+	int copy;
+
+	escaped_name = gnome_vfs_escape_string (base_name);
+
+	path = g_strdup_printf ("%s/%s%s",
+				directory_uri,
+				escaped_name,
+				extension);
+	exists = test_uri_exists (path);
+
+	copy = 1;
+	while (exists) {
+		g_free (path);
+		path = g_strdup_printf ("%s/%s-%d%s",
+					directory_uri,
+					escaped_name,
+					copy,
+					extension);
+
+		exists = test_uri_exists (path);
+
+		copy++;
+	}
+
+	g_free (escaped_name);
+
+	return path;
+}
+
 char *
 nautilus_unique_temporary_file_name (void)
 {
@@ -383,49 +456,27 @@ nautilus_get_vfs_method_display_name (char *method)
 	return NULL;
 }
 
-gboolean
-nautilus_have_broken_filenames (void)
-{
-	static gboolean initialized = FALSE;
-	static gboolean broken;
-  
-	if (initialized) {
-		return broken;
-	}
-
-	broken = g_getenv ("G_BROKEN_FILENAMES") != NULL;
-  
-	initialized = TRUE;
-  
-	return broken;
-}
-
 char *
 nautilus_get_uri_shortname_for_display (GnomeVFSURI *uri)
 {
-	gboolean broken_filenames;
 	char *utf8_name, *name, *tmp;
+	char *text_uri, *local_file;
 	gboolean validated;
 	const char *method;
+
 	
 	validated = FALSE;
 	name = gnome_vfs_uri_extract_short_name (uri);
 	if (name == NULL) {
 		name = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_PASSWORD);
 	} else if (g_ascii_strcasecmp (uri->method_string, "file") == 0) {
-		broken_filenames = nautilus_have_broken_filenames ();
-		if (broken_filenames || !g_utf8_validate (name, -1, NULL)) {
-			utf8_name = g_locale_to_utf8 (name, -1, NULL, NULL, NULL);
-			if (utf8_name != NULL) {
-				g_free (name);
-				name = utf8_name;
-				/* Guaranteed to be correct utf8 here */
-				validated = TRUE;
-			}
-		} else if (!broken_filenames) {
-			/* name was valid, no need to re-validate */
-			validated = TRUE;
-		}
+		text_uri = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_PASSWORD);
+		local_file = gnome_vfs_get_local_path_from_uri (text_uri);
+		g_free (name);
+		name = g_filename_display_basename (local_file);
+		g_free (local_file);
+		g_free (text_uri);
+		validated = TRUE;
 	} else if (!gnome_vfs_uri_has_parent (uri)) {
 		/* Special-case the display name for roots that are not local files */
 		method = nautilus_get_vfs_method_display_name (uri->method_string);
