@@ -29,8 +29,8 @@
 #include "nautilus-directory-private.h"
 #include "nautilus-file.h"
 #include <eel/eel-glib-extensions.h>
-#include <eel/eel-gtk-macros.h>
 #include <gtk/gtksignal.h>
+#include <libgnome/gnome-macros.h>
 
 struct NautilusMergedDirectoryDetails {
 	GList *directories;
@@ -40,12 +40,11 @@ struct NautilusMergedDirectoryDetails {
 };
 
 typedef struct {
-	/* Basic configuration. */
 	NautilusMergedDirectory *merged;
 	NautilusDirectoryCallback callback;
 	gpointer callback_data;
 
-	GList *wait_for_attributes;
+	NautilusFileAttributes wait_for_attributes;
 	gboolean wait_for_file_list;
 
 	GList *non_ready_directories;
@@ -57,7 +56,7 @@ typedef struct {
 
 	gboolean monitor_hidden_files;
 	gboolean monitor_backup_files;
-	GList *monitor_attributes;
+	NautilusFileAttributes monitor_attributes;
 } MergedMonitor;
 
 enum {
@@ -68,53 +67,8 @@ enum {
 
 static guint signals[LAST_SIGNAL];
 
-static void     nautilus_merged_directory_init       (gpointer                 object,
-							    gpointer                 klass);
-static void     nautilus_merged_directory_class_init (gpointer                 klass);
-static void     remove_all_real_directories                (NautilusMergedDirectory *merged);
-static guint    merged_callback_hash                       (gconstpointer            merged_callback);
-static gboolean merged_callback_equal                      (gconstpointer            merged_callback,
-							    gconstpointer            merged_callback_2);
-
-EEL_CLASS_BOILERPLATE (NautilusMergedDirectory,
-				   nautilus_merged_directory,
-				   NAUTILUS_TYPE_DIRECTORY)
-
-static void
-nautilus_merged_directory_init (gpointer object, gpointer klass)
-{
-	NautilusMergedDirectory *merged;
-
-	merged = NAUTILUS_MERGED_DIRECTORY (object);
-
-	merged->details = g_new0 (NautilusMergedDirectoryDetails, 1);
-	merged->details->callbacks = g_hash_table_new
-		(merged_callback_hash, merged_callback_equal);
-	merged->details->monitors = g_hash_table_new (NULL, NULL);
-}
-
-static void
-merged_destroy (GtkObject *object)
-{
-	NautilusMergedDirectory *merged;
-
-	merged = NAUTILUS_MERGED_DIRECTORY (object);
-
-	remove_all_real_directories (merged);
-
-	if (g_hash_table_size (merged->details->callbacks) != 0) {
-		g_warning ("call_when_ready still pending when merged virtual directory is destroyed");
-	}
-	if (g_hash_table_size (merged->details->monitors) != 0) {
-		g_warning ("file monitor still active when merged virtual directory is destroyed");
-	}
-
-	g_hash_table_destroy (merged->details->callbacks);
-	g_hash_table_destroy (merged->details->monitors);
-	g_free (merged->details);
-
-	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
-}
+GNOME_CLASS_BOILERPLATE (NautilusMergedDirectory, nautilus_merged_directory,
+			 NautilusDirectory, NAUTILUS_TYPE_DIRECTORY)
 
 static guint
 merged_callback_hash (gconstpointer merged_callback_as_pointer)
@@ -145,7 +99,6 @@ merged_callback_destroy (MergedCallback *merged_callback)
 	g_assert (merged_callback != NULL);
 	g_assert (NAUTILUS_IS_MERGED_DIRECTORY (merged_callback->merged));
 
-	eel_g_list_free_deep (merged_callback->wait_for_attributes);
 	g_list_free (merged_callback->non_ready_directories);
 	nautilus_file_list_free (merged_callback->merged_file_list);
 	g_free (merged_callback);
@@ -176,8 +129,7 @@ merged_callback_remove_directory (MergedCallback *merged_callback,
 				  NautilusDirectory *directory)
 {
 	merged_callback->non_ready_directories = g_list_remove
-		(merged_callback->non_ready_directories,
-		 directory);
+		(merged_callback->non_ready_directories, directory);
 	merged_callback_check_done (merged_callback);
 }
 
@@ -205,7 +157,7 @@ directory_ready_callback (NautilusDirectory *directory,
 
 static void
 merged_call_when_ready (NautilusDirectory *directory,
-			GList *file_attributes,
+			NautilusFileAttributes file_attributes,
 			gboolean wait_for_file_list,
 			NautilusDirectoryCallback callback,
 			gpointer callback_data)
@@ -229,7 +181,7 @@ merged_call_when_ready (NautilusDirectory *directory,
 	merged_callback->merged = merged;
 	merged_callback->callback = callback;
 	merged_callback->callback_data = callback_data;
-	merged_callback->wait_for_attributes = eel_g_str_list_copy (file_attributes);
+	merged_callback->wait_for_attributes = file_attributes;
 	merged_callback->wait_for_file_list = wait_for_file_list;
 	for (node = merged->details->directories; node != NULL; node = node->next) {
 		merged_callback->non_ready_directories = g_list_prepend
@@ -300,13 +252,13 @@ build_merged_callback_list (NautilusDirectory *directory,
 
 /* Create a monitor on each of the directories in the list. */
 static void
-merged_file_monitor_add (NautilusDirectory *directory,
-			 gconstpointer client,
-			 gboolean monitor_hidden_files,
-			 gboolean monitor_backup_files,
-			 GList *file_attributes,
-			 NautilusDirectoryCallback callback,
-			 gpointer callback_data)
+merged_monitor_add (NautilusDirectory *directory,
+		    gconstpointer client,
+		    gboolean monitor_hidden_files,
+		    gboolean monitor_backup_files,
+		    NautilusFileAttributes file_attributes,
+		    NautilusDirectoryCallback callback,
+		    gpointer callback_data)
 {
 	NautilusMergedDirectory *merged;
 	MergedMonitor *monitor;
@@ -321,7 +273,6 @@ merged_file_monitor_add (NautilusDirectory *directory,
 	monitor = g_hash_table_lookup (merged->details->monitors, client);
 	if (monitor != NULL) {
 		g_assert (monitor->merged == merged);
-		eel_g_list_free_deep (monitor->monitor_attributes);
 	} else {
 		monitor = g_new0 (MergedMonitor, 1);
 		monitor->merged = merged;
@@ -330,7 +281,7 @@ merged_file_monitor_add (NautilusDirectory *directory,
 	}
 	monitor->monitor_hidden_files = monitor_hidden_files;
 	monitor->monitor_backup_files = monitor_backup_files;
-	monitor->monitor_attributes = eel_g_str_list_copy (file_attributes);
+	monitor->monitor_attributes = file_attributes;
 	
 	/* Call through to the real directory add calls. */
 	merged_callback_list = NULL;
@@ -347,14 +298,26 @@ merged_file_monitor_add (NautilusDirectory *directory,
 	nautilus_file_list_free (merged_callback_list);
 }
 
+static void
+merged_monitor_destroy (NautilusMergedDirectory *merged, MergedMonitor *monitor)
+{
+	GList *node;
+
+	/* Call through to the real directory remove calls. */
+	for (node = merged->details->directories; node != NULL; node = node->next) {
+		nautilus_directory_file_monitor_remove (node->data, monitor);
+	}
+
+	g_free (monitor);
+}
+
 /* Remove the monitor from each of the directories in the list. */
 static void
-merged_file_monitor_remove (NautilusDirectory *directory,
-			    gconstpointer client)
+merged_monitor_remove (NautilusDirectory *directory,
+		       gconstpointer client)
 {
 	NautilusMergedDirectory *merged;
 	MergedMonitor *monitor;
-	GList *node;
 	
 	merged = NAUTILUS_MERGED_DIRECTORY (directory);
 	
@@ -365,14 +328,7 @@ merged_file_monitor_remove (NautilusDirectory *directory,
 	}
 	g_hash_table_remove (merged->details->monitors, client);
 
-	/* Call through to the real directory remove calls. */
-	for (node = merged->details->directories; node != NULL; node = node->next) {
-		nautilus_directory_file_monitor_remove
-			(node->data, monitor);
-	}
-
-	eel_g_list_free_deep (monitor->monitor_attributes);
-	g_free (monitor);
+	merged_monitor_destroy (merged, monitor);
 }
 
 static void
@@ -441,6 +397,30 @@ merged_is_not_empty (NautilusDirectory *directory)
 	return FALSE;
 }
 
+static GList *
+merged_get_file_list (NautilusDirectory *directory)
+{
+	GList *dirs_file_list, *merged_dir_file_list;
+	GList *dir_list;
+	GList *cur_node;
+
+	dirs_file_list = NULL;
+	dir_list = NAUTILUS_MERGED_DIRECTORY (directory)->details->directories;
+
+	for (cur_node = dir_list; cur_node != NULL; cur_node = cur_node->next) {
+		NautilusDirectory *cur_dir;
+
+		cur_dir = NAUTILUS_DIRECTORY (cur_node->data);
+		dirs_file_list = g_list_concat (dirs_file_list,
+						 nautilus_directory_get_file_list (cur_dir));
+	}
+
+	merged_dir_file_list = GNOME_CALL_PARENT_WITH_DEFAULT
+				(NAUTILUS_DIRECTORY_CLASS, get_file_list, (directory), NULL);
+
+	return g_list_concat (dirs_file_list, merged_dir_file_list);
+}
+
 static void
 forward_files_added_cover (NautilusDirectory *real_directory,
 			   GList *files,
@@ -500,10 +480,8 @@ merged_add_real_directory (NautilusMergedDirectory *merged,
 	merged->details->directories_not_done_loading = g_list_prepend
 		(merged->details->directories_not_done_loading, real_directory);
 
-	g_signal_connect (real_directory,
-			    "done_loading",
-			    G_CALLBACK (done_loading_callback),
-			    merged);
+	g_signal_connect_object (real_directory, "done_loading",
+				 G_CALLBACK (done_loading_callback), merged, 0);
 
 	/* FIXME bugzilla.gnome.org 45084: The done_loading part won't work for the case where
          * we have no directories in our list.
@@ -515,14 +493,10 @@ merged_add_real_directory (NautilusMergedDirectory *merged,
 			      real_directory);
 	/* FIXME bugzilla.gnome.org 42541: Do we need to add the directory to callbacks too? */
 
-	g_signal_connect (real_directory,
-			    "files_added",
-			    G_CALLBACK (forward_files_added_cover),
-			    merged);
-	g_signal_connect (real_directory,
-			    "files_changed",
-			    G_CALLBACK (forward_files_changed_cover),
-			    merged);
+	g_signal_connect_object (real_directory, "files_added",
+				 G_CALLBACK (forward_files_added_cover), merged, 0);
+	g_signal_connect_object (real_directory, "files_changed",
+				 G_CALLBACK (forward_files_changed_cover), merged, 0);
 }
 
 void
@@ -538,9 +512,7 @@ nautilus_merged_directory_add_real_directory (NautilusMergedDirectory *merged,
 		return;
 	}
 
-	g_signal_emit (merged,
-			 signals[ADD_REAL_DIRECTORY], 0,
-			 real_directory);
+	g_signal_emit (merged, signals[ADD_REAL_DIRECTORY], 0, real_directory);
 }
 
 GList *
@@ -576,17 +548,17 @@ merged_remove_real_directory (NautilusMergedDirectory *merged,
 	g_return_if_fail (g_list_find (merged->details->directories, real_directory) != NULL);
 
 	/* Remove this directory from callbacks and monitors. */
-	eel_g_hash_table_safe_for_each
-		(merged->details->callbacks,
-		 merged_callback_remove_directory_cover,
-		 real_directory);
-	g_hash_table_foreach
-		(merged->details->monitors,
-		 monitor_remove_directory,
-		 real_directory);
+	eel_g_hash_table_safe_for_each (merged->details->callbacks,
+					merged_callback_remove_directory_cover,
+					real_directory);
+	g_hash_table_foreach (merged->details->monitors,
+			      monitor_remove_directory,
+			      real_directory);
 
 	/* Disconnect all the signals. */
-	gtk_signal_disconnect_by_data (GTK_OBJECT (real_directory), merged);
+	g_signal_handlers_disconnect_matched
+		(real_directory, G_SIGNAL_MATCH_DATA,
+		 0, 0, NULL, NULL, merged);
 
 	/* Remove from our list of directories. */
 	merged->details->directories = g_list_remove
@@ -607,61 +579,96 @@ nautilus_merged_directory_remove_real_directory (NautilusMergedDirectory *merged
 		return;
 	}
 
-	g_signal_emit (merged,
-			 signals[REMOVE_REAL_DIRECTORY], 0,
-			 real_directory);
+	g_signal_emit (merged, signals[REMOVE_REAL_DIRECTORY], 0, real_directory);
 }
 
 static void
-remove_all_real_directories (NautilusMergedDirectory *merged)
+merged_monitor_destroy_cover (gpointer key,
+			      gpointer value,
+			      gpointer callback_data)
 {
-	while (merged->details->directories != NULL) {
-		nautilus_merged_directory_remove_real_directory
-			(merged, merged->details->directories->data);
-	}
+	merged_monitor_destroy (callback_data, value);
 }
 
 static void
-nautilus_merged_directory_class_init (gpointer klass)
+merged_callback_destroy_cover (gpointer key,
+			       gpointer value,
+			       gpointer callback_data)
+{
+	merged_callback_destroy (value);
+}
+
+static void
+merged_finalize (GObject *object)
+{
+	NautilusMergedDirectory *merged;
+
+	merged = NAUTILUS_MERGED_DIRECTORY (object);
+
+	g_hash_table_foreach (merged->details->monitors,
+			      merged_monitor_destroy_cover, merged);
+	g_hash_table_foreach (merged->details->callbacks,
+			      merged_callback_destroy_cover, NULL);
+
+	g_hash_table_destroy (merged->details->callbacks);
+	g_hash_table_destroy (merged->details->monitors);
+	nautilus_directory_list_free (merged->details->directories);
+	g_list_free (merged->details->directories_not_done_loading);
+	g_free (merged->details);
+
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+static void
+nautilus_merged_directory_instance_init (NautilusMergedDirectory *merged)
+{
+	merged->details = g_new0 (NautilusMergedDirectoryDetails, 1);
+	merged->details->callbacks = g_hash_table_new
+		(merged_callback_hash, merged_callback_equal);
+	merged->details->monitors = g_hash_table_new (NULL, NULL);
+}
+
+static void
+nautilus_merged_directory_class_init (NautilusMergedDirectoryClass *class)
 {
-	GtkObjectClass *object_class;
 	NautilusDirectoryClass *directory_class;
-	NautilusMergedDirectoryClass *merged_directory_class;
 
-	object_class = GTK_OBJECT_CLASS (klass);
-	directory_class = NAUTILUS_DIRECTORY_CLASS (klass);
-	merged_directory_class = NAUTILUS_MERGED_DIRECTORY_CLASS (klass);
+	directory_class = NAUTILUS_DIRECTORY_CLASS (class);
 	
-	object_class->destroy = merged_destroy;
+	G_OBJECT_CLASS (class)->finalize = merged_finalize;
 
 	directory_class->contains_file = merged_contains_file;
 	directory_class->call_when_ready = merged_call_when_ready;
 	directory_class->cancel_callback = merged_cancel_callback;
-	directory_class->file_monitor_add = merged_file_monitor_add;
-	directory_class->file_monitor_remove = merged_file_monitor_remove;
+	directory_class->file_monitor_add = merged_monitor_add;
+	directory_class->file_monitor_remove = merged_monitor_remove;
 	directory_class->force_reload = merged_force_reload;
  	directory_class->are_all_files_seen = merged_are_all_files_seen;
 	directory_class->is_not_empty = merged_is_not_empty;
+	/* Override get_file_list so that we can return a list that includes
+	 * the files from each of the directories in NautilusMergedDirectory->details->directories.
+         */
+	directory_class->get_file_list = merged_get_file_list;
 
-	merged_directory_class->add_real_directory = merged_add_real_directory;
-	merged_directory_class->remove_real_directory = merged_remove_real_directory;
+	class->add_real_directory = merged_add_real_directory;
+	class->remove_real_directory = merged_remove_real_directory;
 
 	signals[ADD_REAL_DIRECTORY] 
 		= g_signal_new ("add_real_directory",
-		                G_TYPE_FROM_CLASS (object_class),
+		                G_TYPE_FROM_CLASS (class),
 		                G_SIGNAL_RUN_LAST,
 		                G_STRUCT_OFFSET (NautilusMergedDirectoryClass, 
-						     add_real_directory),
+						 add_real_directory),
 		                NULL, NULL,
-		                gtk_marshal_VOID__POINTER,
+		                g_cclosure_marshal_VOID__POINTER,
 		                G_TYPE_NONE, 1, G_TYPE_POINTER);
 	signals[REMOVE_REAL_DIRECTORY] 
 		= g_signal_new ("remove_real_directory",
-		                G_TYPE_FROM_CLASS (object_class),
+		                G_TYPE_FROM_CLASS (class),
 		                G_SIGNAL_RUN_LAST,
 		                G_STRUCT_OFFSET (NautilusMergedDirectoryClass, 
-						     remove_real_directory),
+						 remove_real_directory),
 		                NULL, NULL,
-		                gtk_marshal_VOID__POINTER,
+		                g_cclosure_marshal_VOID__POINTER,
 		                G_TYPE_NONE, 1, G_TYPE_POINTER);
 }
