@@ -27,6 +27,7 @@
 #include "nautilus-file-management-properties.h"
 
 #include <string.h>
+#include <time.h>
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
@@ -55,6 +56,7 @@
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_ICON_VIEW_ZOOM_WIDGET "iconview_zoom_optionmenu"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_LIST_VIEW_ZOOM_WIDGET "listview_zoom_optionmenu"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_SORT_ORDER_WIDGET "sort_order_optionmenu"
+#define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_DATE_FORMAT_WIDGET "date_format_optionmenu"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_PREVIEW_TEXT_WIDGET "preview_text_optionmenu"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_PREVIEW_IMAGE_WIDGET "preview_image_optionmenu"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_PREVIEW_SOUND_WIDGET "preview_sound_optionmenu"
@@ -64,6 +66,7 @@
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_FOLDERS_FIRST_WIDGET "sort_folders_first_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_COMPACT_LAYOUT_WIDGET "compact_layout_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_LABELS_BESIDE_ICONS_WIDGET "labels_beside_icons_checkbutton"
+#define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_ALWAYS_USE_BROWSER_WIDGET "always_use_browser_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_TRASH_CONFIRM_WIDGET "trash_confirm_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_TRASH_DELETE_WIDGET "trash_delete_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_OPEN_NEW_WINDOW_WIDGET "new_window_checkbutton"
@@ -99,6 +102,13 @@ static const char *sort_order_values[] = {
 	NULL
 };
 
+static const char *date_format_values[] = {
+	"locale",
+	"iso",
+	"informal",
+	NULL
+};
+
 static const char *preview_values[] = {
 	"always",
 	"local_only",
@@ -452,6 +462,43 @@ nautilus_file_management_properties_dialog_setup_icon_caption_page (GladeXML *xm
 	update_icon_captions_from_gconf (xml_dialog);
 }
 
+static void
+create_date_format_menu (GladeXML *xml_dialog)
+{
+	GtkWidget *option_menu;
+	GtkWidget *menu;
+	GtkWidget *menu_item;
+	gchar *date_string;
+	time_t now_raw;
+	struct tm* now;
+	option_menu = glade_xml_get_widget (xml_dialog,
+					    NAUTILUS_FILE_MANAGEMENT_PROPERTIES_DATE_FORMAT_WIDGET);
+	menu = gtk_menu_new ();
+
+	now_raw = time (NULL);
+	now = localtime (&now_raw);
+
+	date_string = eel_strdup_strftime ("%c", now);
+	menu_item = gtk_menu_item_new_with_label (date_string);
+	g_free (date_string);
+	gtk_widget_show (menu_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+
+	date_string = eel_strdup_strftime ("%Y-%m-%d %H:%M:%S", now);
+	menu_item = gtk_menu_item_new_with_label (date_string);
+	g_free (date_string);
+	gtk_widget_show (menu_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+
+	date_string = eel_strdup_strftime (_("today at %-I:%M:%S %p"), now);
+	menu_item = gtk_menu_item_new_with_label (date_string);
+	g_free (date_string);
+	gtk_widget_show (menu_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
+}
+
 static void
 set_columns_from_gconf (NautilusColumnChooser *chooser)
 {
@@ -524,6 +571,7 @@ nautilus_file_management_properties_dialog_setup (GladeXML *xml_dialog, GtkWindo
 	nautilus_file_management_properties_size_group_create (xml_dialog,
 							       "preview_label",
 							       5);
+	create_date_format_menu (xml_dialog);
 
 	/* setup preferences */
 	eel_preferences_glade_connect_bool (xml_dialog,
@@ -535,6 +583,9 @@ nautilus_file_management_properties_dialog_setup (GladeXML *xml_dialog, GtkWindo
 	eel_preferences_glade_connect_bool (xml_dialog,
 					    NAUTILUS_FILE_MANAGEMENT_PROPERTIES_FOLDERS_FIRST_WIDGET,
 					    NAUTILUS_PREFERENCES_SORT_DIRECTORIES_FIRST); 
+	eel_preferences_glade_connect_bool (xml_dialog,
+					    NAUTILUS_FILE_MANAGEMENT_PROPERTIES_ALWAYS_USE_BROWSER_WIDGET,
+					    NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER);
 	eel_preferences_glade_connect_bool (xml_dialog,
 					    NAUTILUS_FILE_MANAGEMENT_PROPERTIES_TRASH_CONFIRM_WIDGET,
 					    NAUTILUS_PREFERENCES_CONFIRM_TRASH);
@@ -586,6 +637,10 @@ nautilus_file_management_properties_dialog_setup (GladeXML *xml_dialog, GtkWindo
 							       NAUTILUS_FILE_MANAGEMENT_PROPERTIES_PREVIEW_FOLDER_WIDGET,
 							       NAUTILUS_PREFERENCES_SHOW_DIRECTORY_ITEM_COUNTS,
 							       preview_values);
+	eel_preferences_glade_connect_string_enum_option_menu (xml_dialog,
+							       NAUTILUS_FILE_MANAGEMENT_PROPERTIES_DATE_FORMAT_WIDGET,
+							       NAUTILUS_PREFERENCES_DATE_FORMAT,
+							       date_format_values);
 
 	eel_preferences_glade_connect_string_enum_radio_button (xml_dialog,
 								click_behavior_components,
